<?php
/**
 * Grider
 * Grider - Creating a dynamic grid layout has never been easier.
 * Exclusively on https://1.envato.market/grider-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.5
 * @copyright       (C) 2018 - 2022 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\GriderElementor;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

use Merkulove\GriderElementor\Unity\Plugin;
use WP_REST_Server;

final class Rest {

	/**
	 * The one true Rest.
	 *
     * @access private
	 * @var Rest
	 **/
	private static $instance;

	/**
	 * @return void
	 */
    public function setup() {

	    /** Rest api route for get layouts list */
	    add_action( 'rest_api_init', function () {

		    register_rest_route(

				untrailingslashit( Plugin::get_slug() . '/v1' ),
				'/(?P<action>\w+)/',
				array(
				    'methods' => WP_REST_Server::ALLMETHODS,
				    'callback' => [ $this, 'rest_callback' ],
				    'permission_callback' => '__return_true',
			    )
		    );

	    });

    }

	/**
	 * @param $request
	 *
	 * @return int[]|\WP_Post[]
	 */
	public function rest_callback( $request ){

		// Get and store REST action
		$params = $request->get_params();

		$posts = get_posts( $this->get_posts_args( $params ) );

		$res = array();

		foreach ( $posts as $post ) {

			$featured_image = strpos( get_post_mime_type( $post->ID ), 'image' ) !== false ?
				wp_get_attachment_image_url( $post->ID, 'full' ) : get_the_post_thumbnail_url( $post->ID );

			$res[] = array(

				'id' => $post->ID,
				'permalink' => $post->guid,
				'title' => esc_html( $post->post_title ),
				'description' => get_the_excerpt( $post->ID ),
				'featuredImage' => $featured_image

			);

		}

		return $res;

	}

	private function get_posts_args( $params ) {

		if ( ! isset( $params[ 'action' ] ) || $params[ 'action' ] !== 'posts' ) { return array(); }

		$amount = $params[ 'per_page' ];
		$type = $params[ 'type' ];
		$order_by = $params[ 'orderby' ];
		$order = $params[ 'order' ];

		$args = array(
			'numberposts'       => $amount,
			'posts_per_page'    => -1,
			'post_type'         => $type,
			'orderby'           => $order_by,
			'order'             => $order,
		);

		/** Merge category to query arguments */
		if ( isset( $params[ 'categories' ] ) ) {

			$categories = explode( ',', $params[ 'categories' ] );

			$args = array_merge( $args,
				array(
					'category' => $categories,
				)
			);

		}

		/** Merge taxonomy to query arguments */
		if ( isset( $params[ 'taxonomy' ] ) && $params[ 'taxonomy' ] !== '' ) {

			if ( isset( $params[ 'terms' ] ) && $params[ 'terms' ] !== '' ) {

				// Get terms query
				$terms_query = array();
				$terms = explode( ',', $params[ 'terms' ] );
				foreach ( $terms as $term ) {

					$terms_query[] = array(

						'taxonomy' => $params[ 'taxonomy' ],
						'field' => 'term_id',
						'terms' => intval( $term ),

					);

				}

				// Get taxonomy query
				$tax_query = array(
					'relation' => $params[ 'relation' ],
				);
				$tax_query = array_merge( $tax_query, $terms_query );

				// Merge arguments
				$args = array_merge( $args,
					array(
						'tax_query' => $tax_query
					)
				);

			} else {

				// Merge arguments
				$args = array_merge( $args,
					array(
						'tax_query' => array(
							array(

								'taxonomy' => $params[ 'taxonomy' ],
								'operator' => 'EXISTS',

							)
						)
					)
				);

			}

		}

		return $args;

	}

	/**
	 * Main Rest Instance.
	 * Insures that only one instance of Rest exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return Rest
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
